@@ -18,7 +18,7 @@
 #pragma implementation				// gcc: Class implementation
 #endif
 
-#include <my_global.h>
+#include "mariadb.h"
 #include "sql_priv.h"
 #include "sql_select.h"
 #include "my_json_writer.h"
@@ -29,6 +29,14 @@ const char * STR_DELETING_ALL_ROWS= "Deleting all rows";
 const char * STR_IMPOSSIBLE_WHERE= "Impossible WHERE";
 const char * STR_NO_ROWS_AFTER_PRUNING= "No matching rows after partition pruning";
 
+const char *unit_operation_text[4]=
+{
+   "UNIT RESULT","UNION RESULT","INTERSECT RESULT","EXCEPT RESULT"
+};
+
+const char *pushed_derived_text= "PUSHED DERIVED";
+const char *pushed_select_text= "PUSHED SELECT";
+
 static void write_item(Json_writer *writer, Item *item);
 static void append_item_to_str(String *out, Item *item);
 
@@ -228,7 +236,7 @@ void Explain_query::print_explain_json(select_result_sink *output,
 
   CHARSET_INFO *cs= system_charset_info;
   List<Item> item_list;
-  String *buf= &writer.output;
+  const String *buf= writer.output.get_string();
   item_list.push_back(new (thd->mem_root)
                       Item_string(thd, buf->ptr(), buf->length(), cs),
                       thd->mem_root);
@@ -288,7 +296,7 @@ static void push_string_list(THD *thd, List<Item> *item_list,
     else
       buf->append(',');
 
-    buf->append(line);
+    buf->append(line, strlen(line));
   }
   push_string(thd, item_list, buf);
 }
@@ -329,6 +337,9 @@ int print_explain_row(select_result_sink *result,
   List<Item> item_list;
   Item *item;
 
+  if (!select_type[0])
+    return 0;
+
   item_list.push_back(new (mem_root) Item_int(thd, (int32) select_number),
                       mem_root);
   item_list.push_back(new (mem_root) Item_string_sys(thd, select_type),
@@ -375,21 +386,31 @@ int print_explain_row(select_result_sink *result,
   item_list.push_back(item, mem_root);
 
   /* 'rows' */
+  StringBuffer<64> rows_str;
   if (rows)
   {
+    rows_str.append_ulonglong((ulonglong)(*rows));
     item_list.push_back(new (mem_root)
-                        Item_int(thd, *rows, MY_INT64_NUM_DECIMAL_DIGITS),
-                        mem_root);
+                        Item_string_sys(thd, rows_str.ptr(),
+                                        rows_str.length()), mem_root);
   }
   else
     item_list.push_back(item_null, mem_root);
   
   /* 'r_rows' */
+  StringBuffer<64> r_rows_str;
   if (is_analyze)
   {
     if (r_rows)
-      item_list.push_back(new (mem_root) Item_float(thd, *r_rows, 2),
-                          mem_root);
+    {
+      Item_float *fl= new (mem_root) Item_float(thd, *r_rows, 2);
+      String tmp;
+      String *res= fl->val_str(&tmp);
+      r_rows_str.append(*res);
+      item_list.push_back(new (mem_root)
+                          Item_string_sys(thd, r_rows_str.ptr(),
+                                          r_rows_str.length()), mem_root);
+    }
     else
       item_list.push_back(item_null, mem_root);
   }
@@ -421,14 +442,34 @@ int print_explain_row(select_result_sink *result,
 uint Explain_union::make_union_table_name(char *buf)
 {
   uint childno= 0;
-  uint len= 6, lastop= 0;
-  memcpy(buf, STRING_WITH_LEN("<union"));
+  uint len, lastop= 0;
+  LEX_CSTRING type;
+  switch (operation)
+  {
+    case OP_MIX:
+      lex_string_set3(&type, STRING_WITH_LEN("<unit"));
+      break;
+    case OP_UNION:
+      lex_string_set3(&type, STRING_WITH_LEN("<union"));
+      break;
+    case OP_INTERSECT:
+      lex_string_set3(&type, STRING_WITH_LEN("<intersect"));
+      break;
+    case OP_EXCEPT:
+      lex_string_set3(&type, STRING_WITH_LEN("<except"));
+      break;
+    default:
+      DBUG_ASSERT(0);
+      type.str= NULL;
+      type.length= 0;
+  }
+  memcpy(buf, type.str, (len= (uint)type.length));
 
   for (; childno < union_members.elements() && len + lastop + 5 < NAME_LEN;
        childno++)
   {
     len+= lastop;
-    lastop= my_snprintf(buf + len, NAME_LEN - len,
+    lastop= (uint)my_snprintf(buf + len, NAME_LEN - len,
                         "%u,", union_members.at(childno));
   }
 
@@ -465,7 +506,7 @@ int Explain_union::print_explain(Explain_query *query,
   if (!using_tmp)
     return 0;
 
-  /* Print a line with "UNION RESULT" */
+  /* Print a line with "UNIT RESULT" */
   List<Item> item_list;
   Item *item_null= new (mem_root) Item_null(thd);
 
@@ -504,10 +545,17 @@ int Explain_union::print_explain(Explain_query *query,
   item_list.push_back(item_null, mem_root);
   
   /* `r_rows` */
+  StringBuffer<64> r_rows_str;
   if (is_analyze)
   {
     double avg_rows= fake_select_lex_tracker.get_avg_rows();
-    item_list.push_back(new (mem_root) Item_float(thd, avg_rows, 2), mem_root);
+    Item_float *fl= new (mem_root) Item_float(thd, avg_rows, 2);
+    String tmp;
+    String *res= fl->val_str(&tmp);
+    r_rows_str.append(*res);
+    item_list.push_back(new (mem_root)
+                        Item_string_sys(thd, r_rows_str.ptr(),
+                                        r_rows_str.length()), mem_root);
   }
 
   /* `filtered` */
@@ -556,27 +604,29 @@ void Explain_union::print_explain_json(Explain_query *query,
   else
     writer->add_member("union_result").start_object();
 
-  // using_temporary_table
-  make_union_table_name(table_name_buffer);
-  writer->add_member("table_name").add_str(table_name_buffer);
-  writer->add_member("access_type").add_str("ALL"); // not very useful
-
-  /* r_loops (not present in tabular output) */
-  if (is_analyze)
+  if (using_tmp)
   {
-    writer->add_member("r_loops").add_ll(fake_select_lex_tracker.get_loops());
-  }
+    make_union_table_name(table_name_buffer);
+    writer->add_member("table_name").add_str(table_name_buffer);
+    writer->add_member("access_type").add_str("ALL"); // not very useful
 
-  /* `r_rows` */
-  if (is_analyze)
-  {
-    writer->add_member("r_rows");
-    if (fake_select_lex_tracker.has_scans())
-      writer->add_double(fake_select_lex_tracker.get_avg_rows());
-    else
-      writer->add_null();
-  }
+    /* r_loops (not present in tabular output) */
+    if (is_analyze)
+    {
+      writer->add_member("r_loops").add_ll(
+          fake_select_lex_tracker.get_loops());
+    }
 
+    /* `r_rows` */
+    if (is_analyze)
+    {
+      writer->add_member("r_rows");
+      if (fake_select_lex_tracker.has_scans())
+        writer->add_double(fake_select_lex_tracker.get_avg_rows());
+      else
+        writer->add_null();
+    }
+  }
   writer->add_member("query_specifications").start_array();
 
   for (int i= 0; i < (int) union_members.elements(); i++)
@@ -723,7 +773,15 @@ int Explain_select::print_explain(Explain_query *query,
   THD *thd= output->thd;
   MEM_ROOT *mem_root= thd->mem_root;
 
-  if (message)
+  if (select_type == pushed_derived_text || select_type == pushed_select_text)
+  {
+     print_explain_message_line(output, explain_flags, is_analyze,
+                                select_id /*select number*/,
+                                select_type,
+                                NULL, /* rows */
+                                NULL);
+  }
+  else if (message)
   {
     List<Item> item_list;
     Item *item_null= new (mem_root) Item_null(thd);
@@ -817,6 +875,28 @@ int Explain_basic_join::print_explain(Explain_query *query,
 }
 
 
+void Explain_select::add_linkage(Json_writer *writer)
+{
+  const char *operation= NULL;
+  switch (linkage)
+  {
+     case UNION_TYPE:
+       operation= "UNION";
+       break;
+     case INTERSECT_TYPE:
+       operation= "INTERSECT";
+       break;
+     case EXCEPT_TYPE:
+       operation= "EXCEPT";
+       break;
+     default:
+       // It is the first or the only SELECT => no operation
+       break;
+  }
+  if (operation)
+    writer->add_member("operation").add_str(operation);
+}
+
 void Explain_select::print_explain_json(Explain_query *query, 
                                         Json_writer *writer, bool is_analyze)
 {
@@ -824,13 +904,20 @@ void Explain_select::print_explain_json(Explain_query *query,
   
   bool started_cache= print_explain_json_cache(writer, is_analyze);
 
-  if (message)
+  if (message ||
+      select_type == pushed_derived_text ||
+      select_type == pushed_select_text)
   {
     writer->add_member("query_block").start_object();
     writer->add_member("select_id").add_ll(select_id);
+    add_linkage(writer);
 
     writer->add_member("table").start_object();
-    writer->add_member("message").add_str(message);
+    writer->add_member("message").add_str(select_type == pushed_derived_text ?
+                                          "Pushed derived" :
+                                          select_type == pushed_select_text ?
+                                          "Pushed select" :
+                                          message);
     writer->end_object();
 
     print_explain_json_for_children(query, writer, is_analyze);
@@ -840,6 +927,7 @@ void Explain_select::print_explain_json(Explain_query *query,
   {
     writer->add_member("query_block").start_object();
     writer->add_member("select_id").add_ll(select_id);
+    add_linkage(writer);
 
     if (is_analyze && time_tracker.get_loops())
     {
@@ -957,11 +1045,11 @@ void Explain_aggr_filesort::print_json_members(Json_writer *writer,
       first= false;
     else
     {
-      str.append(", ");
+      str.append(STRING_WITH_LEN(", "));
     }
     append_item_to_str(&str, item);
     if (*direction == ORDER::ORDER_DESC)
-      str.append(" desc");
+      str.append(STRING_WITH_LEN(" desc"));
   }
 
   writer->add_member("sort_key").add_str(str.c_ptr_safe());
@@ -1039,14 +1127,15 @@ void Explain_table_access::fill_key_str(String *key_str, bool is_json) const
   CHARSET_INFO *cs= system_charset_info;
   bool is_hj= (type == JT_HASH || type == JT_HASH_NEXT || 
                type == JT_HASH_RANGE || type == JT_HASH_INDEX_MERGE);
-  const char *hash_key_prefix= "#hash#";
+  LEX_CSTRING hash_key_prefix= { STRING_WITH_LEN("#hash#") };
+  const char *key_name;
 
-  if (key.get_key_name())
+  if ((key_name= key.get_key_name()))
   {
     if (is_hj)
-      key_str->append(hash_key_prefix, strlen(hash_key_prefix), cs);
+      key_str->append(hash_key_prefix.str, hash_key_prefix.length, cs);
 
-    key_str->append(key.get_key_name());
+    key_str->append(key_name, strlen(key_name));
 
     if (is_hj && type != JT_HASH)
       key_str->append(':');
@@ -1062,7 +1151,10 @@ void Explain_table_access::fill_key_str(String *key_str, bool is_json) const
     key_str->append(buf2);
   }
   if (type == JT_HASH_NEXT)
-    key_str->append(hash_next_key.get_key_name());
+  {
+    key_name= hash_next_key.get_key_name();
+    key_str->append(key_name, strlen(key_name));
+  }
 }
 
 
@@ -1071,12 +1163,14 @@ void Explain_table_access::fill_key_str(String *key_str, bool is_json) const
    - this is just used key length for ref/range 
    - for index_merge, it is a comma-separated list of lengths.
    - for hash join, it is key_len:pseudo_key_len
+   - [tabular form only] rowid filter length is added after "|".
 
-  The column looks identical in tabular and json forms. In JSON, we consider
-  the column legacy, it is superceded by used_key_parts.
+  In JSON, we consider this column to be legacy, it is superceded by
+  used_key_parts.
 */
 
-void Explain_table_access::fill_key_len_str(String *key_len_str) const
+void Explain_table_access::fill_key_len_str(String *key_len_str,
+                                            bool is_json) const
 {
   bool is_hj= (type == JT_HASH || type == JT_HASH_NEXT || 
                type == JT_HASH_RANGE || type == JT_HASH_INDEX_MERGE);
@@ -1104,35 +1198,48 @@ void Explain_table_access::fill_key_len_str(String *key_len_str) const
     length= longlong10_to_str(hash_next_key.get_key_len(), buf, 10) - buf;
     key_len_str->append(buf, length);
   }
+
+  if (!is_json && rowid_filter)
+  {
+    key_len_str->append('|');
+    StringBuffer<64> filter_key_len;
+    rowid_filter->quick->print_key_len(&filter_key_len);
+    key_len_str->append(filter_key_len);
+  }
 }
 
 
-void Explain_index_use::set(MEM_ROOT *mem_root, KEY *key, uint key_len_arg)
+bool Explain_index_use::set(MEM_ROOT *mem_root, KEY *key, uint key_len_arg)
 {
-  set_pseudo_key(mem_root, key->name);
+  if (set_pseudo_key(mem_root, key->name.str))
+    return 1;
+
   key_len= key_len_arg;
   uint len= 0;
   for (uint i= 0; i < key->usable_key_parts; i++)
   {
-    key_parts_list.append_str(mem_root, key->key_part[i].field->field_name);
+    if (!key_parts_list.append_str(mem_root,
+                                   key->key_part[i].field->field_name.str))
+      return 1;
     len += key->key_part[i].store_length;
     if (len >= key_len_arg)
       break;
   }
+  return 0;
 }
 
 
-void Explain_index_use::set_pseudo_key(MEM_ROOT *root, const char* key_name_arg)
+bool Explain_index_use::set_pseudo_key(MEM_ROOT *root, const char* key_name_arg)
 {
   if (key_name_arg)
   {
-    size_t name_len= strlen(key_name_arg);
-    if ((key_name= (char*)alloc_root(root, name_len+1)))
-      memcpy(key_name, key_name_arg, name_len+1);
+    if (!(key_name= strdup_root(root, key_name_arg)))
+      return 1;
   }
   else
     key_name= NULL;
   key_len= ~(uint) 0;
+  return 0;
 }
 
 
@@ -1184,7 +1291,18 @@ int Explain_table_access::print_explain(select_result_sink *output, uint8 explai
   }
 
   /* `type` column */
-  push_str(thd, &item_list, join_type_str[type]);
+   StringBuffer<64> join_type_buf;
+  if (rowid_filter == NULL)
+    push_str(thd, &item_list, join_type_str[type]);
+  else
+  {
+    join_type_buf.append(join_type_str[type], strlen(join_type_str[type]));
+    join_type_buf.append(STRING_WITH_LEN("|filter"));
+    item_list.push_back(new (mem_root)
+                        Item_string_sys(thd, join_type_buf.ptr(),
+                                             join_type_buf.length()),
+                        mem_root);
+  }
 
   /* `possible_keys` column */
   StringBuffer<64> possible_keys_buf;
@@ -1196,6 +1314,14 @@ int Explain_table_access::print_explain(select_result_sink *output, uint8 explai
   /* `key` */
   StringBuffer<64> key_str;
   fill_key_str(&key_str, false);
+
+  if (rowid_filter)
+  {
+    key_str.append('|');
+    StringBuffer<64> rowid_key_str;
+    rowid_filter->quick->print_key(&rowid_key_str);
+    key_str.append(rowid_key_str);
+  }
   
   if (key_str.length() > 0)
     push_string(thd, &item_list, &key_str);
@@ -1204,7 +1330,7 @@ int Explain_table_access::print_explain(select_result_sink *output, uint8 explai
 
   /* `key_len` */
   StringBuffer<64> key_len_str;
-  fill_key_len_str(&key_len_str);
+  fill_key_len_str(&key_len_str, false);
 
   if (key_len_str.length() > 0)
     push_string(thd, &item_list, &key_len_str);
@@ -1227,17 +1353,27 @@ int Explain_table_access::print_explain(select_result_sink *output, uint8 explai
     push_string_list(thd, &item_list, ref_list, &ref_list_buf);
  
   /* `rows` */
+  StringBuffer<64> rows_str;
   if (rows_set)
   {
+    rows_str.append_ulonglong((ulonglong)rows);
+
+    if (rowid_filter)
+    {
+      rows_str.append(STRING_WITH_LEN(" ("));
+      rows_str.append_ulonglong((ulonglong) (round(rowid_filter->selectivity *
+						   100.0)));
+      rows_str.append(STRING_WITH_LEN("%)"));
+    }
     item_list.push_back(new (mem_root)
-                        Item_int(thd, (ulonglong) rows,
-                                 MY_INT64_NUM_DECIMAL_DIGITS),
-                        mem_root);
+                        Item_string_sys(thd, rows_str.ptr(),
+                                        rows_str.length()), mem_root);
   }
   else
     item_list.push_back(item_null, mem_root);
 
   /* `r_rows` */
+  StringBuffer<64> r_rows_str;
   if (is_analyze)
   {
     if (!tracker.has_scans())
@@ -1247,8 +1383,20 @@ int Explain_table_access::print_explain(select_result_sink *output, uint8 explai
     else
     {
       double avg_rows= tracker.get_avg_rows();
-      item_list.push_back(new (mem_root) Item_float(thd, avg_rows, 2),
-                          mem_root);
+      Item_float *fl= new (mem_root) Item_float(thd, avg_rows, 2);
+      String tmp;
+      String *res= fl->val_str(&tmp);
+      r_rows_str.append(*res);
+      if (rowid_filter)
+      {
+        r_rows_str.append(STRING_WITH_LEN(" ("));
+        r_rows_str.append_ulonglong(
+	  (ulonglong) (rowid_filter->tracker->get_r_selectivity_pct() * 100.0));
+        r_rows_str.append(STRING_WITH_LEN("%)"));
+      }
+      item_list.push_back(new (mem_root)
+                          Item_string_sys(thd, r_rows_str.ptr(),
+                                          r_rows_str.length()), mem_root);
     }
   }
 
@@ -1312,6 +1460,15 @@ int Explain_table_access::print_explain(select_result_sink *output, uint8 explai
     extra_buf.append(STRING_WITH_LEN("Using filesort"));
   }
 
+  if (rowid_filter)
+  {
+    if (first)
+      first= false;
+    else
+      extra_buf.append(STRING_WITH_LEN("; "));
+    extra_buf.append(STRING_WITH_LEN("Using rowid filter"));
+  }
+
   item_list.push_back(new (mem_root)
                       Item_string_sys(thd, extra_buf.ptr(),
                                       extra_buf.length()),
@@ -1484,6 +1641,9 @@ void Explain_table_access::tag_to_json(Json_writer *writer, enum explain_extra_t
     case ET_DISTINCT:
       writer->add_member("distinct").add_bool(true);
       break;
+    case ET_TABLE_FUNCTION:
+      writer->add_member("table_function").add_str("json_table");
+      break;
 
     default:
       DBUG_ASSERT(0);
@@ -1500,6 +1660,29 @@ void add_json_keyset(Json_writer *writer, const char *elem_name,
 }
 
 
+void Explain_rowid_filter::print_explain_json(Explain_query *query,
+                                              Json_writer *writer,
+                                              bool is_analyze)
+{
+  Json_writer_nesting_guard guard(writer);
+  writer->add_member("rowid_filter").start_object();
+  quick->print_json(writer);
+  writer->add_member("rows").add_ll(rows);
+  writer->add_member("selectivity_pct").add_double(selectivity * 100.0);
+  if (is_analyze)
+  {
+    writer->add_member("r_rows").add_double(tracker->get_container_elements());
+    writer->add_member("r_selectivity_pct").
+      add_double(tracker->get_r_selectivity_pct() * 100.0);
+    writer->add_member("r_buffer_size").
+      add_double((double) (tracker->get_container_buff_size()));
+    writer->add_member("r_filling_time_ms").
+      add_double(tracker->get_time_fill_container_ms());
+  }
+  writer->end_object(); // rowid_filter
+}
+
+
 void Explain_table_access::print_explain_json(Explain_query *query,
                                               Json_writer *writer,
                                               bool is_analyze)
@@ -1572,7 +1755,7 @@ void Explain_table_access::print_explain_json(Explain_query *query,
 
   /* `key_length` */
   StringBuffer<64> key_len_str;
-  fill_key_len_str(&key_len_str);
+  fill_key_len_str(&key_len_str, true);
   if (key_len_str.length())
     writer->add_member("key_length").add_str(key_len_str);
 
@@ -1597,6 +1780,11 @@ void Explain_table_access::print_explain_json(Explain_query *query,
   if (!ref_list.is_empty())
     print_json_array(writer, "ref", ref_list);
 
+  if (rowid_filter)
+  {
+    rowid_filter->print_explain_json(query, writer, is_analyze);
+  }
+
   /* r_loops (not present in tabular output) */
   if (is_analyze)
   {
@@ -1629,11 +1817,14 @@ void Explain_table_access::print_explain_json(Explain_query *query,
 
     if (op_tracker.get_loops())
     {
-      writer->add_member("r_total_time_ms").
-              add_double(op_tracker.get_time_ms());
+      double total_time= op_tracker.get_time_ms();
+      if (rowid_filter)
+        total_time+= rowid_filter->tracker->get_time_fill_container_ms();
+      writer->add_member("r_table_time_ms").add_double(total_time);
+      writer->add_member("r_other_time_ms").add_double(extra_time_tracker.get_time_ms());
     }
   }
-  
+
   /* `filtered` */
   if (filtered_set)
     writer->add_member("filtered").add_double(filtered);
@@ -1702,6 +1893,11 @@ void Explain_table_access::print_explain_json(Explain_query *query,
     /* This is a derived table. Print its contents here */
     writer->add_member("materialized").start_object();
     Explain_node *node= query->get_node(derived_select_number);
+    if (node->get_type() == Explain_node::EXPLAIN_SELECT &&
+        ((Explain_select*)node)->is_lateral)
+    {
+      writer->add_member("lateral").add_ll(1);
+    }
     node->print_explain_json(query, writer, is_analyze);
     writer->end_object();
   }
@@ -1735,46 +1931,45 @@ void Explain_table_access::print_explain_json(Explain_query *query,
 
 
 /*
-  Elements in this array match members of enum Extra_tag, defined in
+  Elements in this array match members of enum explain_extra_tag, defined in
   sql_explain.h
 */
 
-const char * extra_tag_text[]=
+const LEX_CSTRING extra_tag_text[]=
 {
-  "ET_none",
-  "Using index condition",
-  "Using index condition(BKA)",
-  "Using ", // special handling
-  "Range checked for each record (index map: 0x", // special handling
-  "Using where with pushed condition",
-  "Using where",
-  "Not exists",
+  { STRING_WITH_LEN("ET_none") },
+  { STRING_WITH_LEN("Using index condition") },
+  { STRING_WITH_LEN("Using index condition(BKA)") },
+  { STRING_WITH_LEN("Using ") }, // special handling
+  { STRING_WITH_LEN("Range checked for each record (index map: 0x") },  // special handling
+  { STRING_WITH_LEN("Using where with pushed condition") },
+  { STRING_WITH_LEN("Using where") },
+  { STRING_WITH_LEN("Not exists") },
   
-  "Using index",
-  "Full scan on NULL key",
-  "Skip_open_table",
-  "Open_frm_only",
-  "Open_full_table", 
+  { STRING_WITH_LEN("Using index") },
+  { STRING_WITH_LEN("Full scan on NULL key") },
+  { STRING_WITH_LEN("Skip_open_table") },
+  { STRING_WITH_LEN("Open_frm_only") },
+  { STRING_WITH_LEN("Open_full_table") },
 
-  "Scanned 0 databases",
-  "Scanned 1 database",
-  "Scanned all databases",
+  { STRING_WITH_LEN("Scanned 0 databases") },
+  { STRING_WITH_LEN("Scanned 1 database") },
+  { STRING_WITH_LEN("Scanned all databases") },
 
-  "Using index for group-by", // special handling
+  { STRING_WITH_LEN("Using index for group-by") }, // special handling
+  { STRING_WITH_LEN("USING MRR: DONT PRINT ME") }, // special handling
 
-  "USING MRR: DONT PRINT ME", // special handling
+  { STRING_WITH_LEN("Distinct") },
+  { STRING_WITH_LEN("LooseScan") },
+  { STRING_WITH_LEN("Start temporary") },
+  { STRING_WITH_LEN("End temporary") },
+  { STRING_WITH_LEN("FirstMatch") },               // special handling
 
-  "Distinct",
-  "LooseScan",
-  "Start temporary",
-  "End temporary",
-  "FirstMatch", // special handling
+  { STRING_WITH_LEN("Using join buffer") },        // special handling
 
-  "Using join buffer", // special handling 
-
-  "const row not found",
-  "unique row not found",
-  "Impossible ON condition"
+  { STRING_WITH_LEN("Const row not found") },
+  { STRING_WITH_LEN("Unique row not found") },
+  { STRING_WITH_LEN("Impossible ON condition") }
 };
 
 
@@ -1794,7 +1989,8 @@ void Explain_table_access::append_tag_name(String *str, enum explain_extra_tag t
       char buf[MAX_KEY / 4 + 1];
       str->append(STRING_WITH_LEN("Range checked for each "
                                    "record (index map: 0x"));
-      str->append(range_checked_fer->keys_map.print(buf));
+      range_checked_fer->keys_map.print(buf);
+      str->append(buf, strlen(buf));
       str->append(')');
       break;
     }
@@ -1808,12 +2004,16 @@ void Explain_table_access::append_tag_name(String *str, enum explain_extra_tag t
       str->append(extra_tag_text[tag]);
 
       str->append(STRING_WITH_LEN(" ("));
-      const char *buffer_type= bka_type.incremental ? "incremental" : "flat";
+      LEX_CSTRING buffer_type;
+      if (bka_type.incremental)
+        buffer_type= { STRING_WITH_LEN("incremental") };
+      else
+        buffer_type= { STRING_WITH_LEN("flat") };
       str->append(buffer_type);
       str->append(STRING_WITH_LEN(", "));
-      str->append(bka_type.join_alg);
+      str->append(bka_type.join_alg, strlen(bka_type.join_alg));
       str->append(STRING_WITH_LEN(" join"));
-      str->append(STRING_WITH_LEN(")"));
+      str->append(')');
       if (bka_type.mrr_type.length())
       {
         str->append(STRING_WITH_LEN("; "));
@@ -1826,9 +2026,9 @@ void Explain_table_access::append_tag_name(String *str, enum explain_extra_tag t
     {
       if (firstmatch_table_name.length())
       {
-        str->append("FirstMatch(");
+        str->append(STRING_WITH_LEN("FirstMatch("));
         str->append(firstmatch_table_name);
-        str->append(")");
+        str->append(')');
       }
       else
         str->append(extra_tag_text[tag]);
@@ -1838,7 +2038,10 @@ void Explain_table_access::append_tag_name(String *str, enum explain_extra_tag t
     {
       str->append(extra_tag_text[tag]);
       if (loose_scan_is_scanning)
-        str->append(" (scanning)");
+        str->append(STRING_WITH_LEN(" (scanning)"));
+      break;
+    case ET_TABLE_FUNCTION:
+      str->append(STRING_WITH_LEN("Table function: json_table"));
       break;
     }
     default:
@@ -1893,13 +2096,16 @@ void Explain_quick_select::print_json(Json_writer *writer)
 
 void Explain_quick_select::print_extra_recursive(String *str)
 {
+  const char *name;
   if (is_basic())
   {
-    str->append(range.get_key_name());
+    name= range.get_key_name();
+    str->append(name, strlen(name));
   }
   else
   {
-    str->append(get_name_by_type());
+    name= get_name_by_type();
+    str->append(name, strlen(name));
     str->append('(');
     List_iterator_fast<Explain_quick_select> it (children);
     Explain_quick_select* child;
@@ -1948,7 +2154,7 @@ void Explain_quick_select::print_key(String *str)
   {
     if (str->length() > 0)
       str->append(',');
-    str->append(range.get_key_name());
+    str->append(range.get_key_name(), strlen(range.get_key_name()));
   }
   else
   {
@@ -2413,7 +2619,11 @@ int Explain_range_checked_fer::append_possible_keys_stat(MEM_ROOT *alloc,
   for (j= 0; j < table->s->keys; j++)
   {
     if (possible_keys.is_set(j))
-      keys_stat_names[j]= key_set.append_str(alloc, table->key_info[j].name);
+    {
+      if (!(keys_stat_names[j]= key_set.append_str(alloc,
+                                                   table->key_info[j].name.str)))
+        return 1;
+    }
     else
       keys_stat_names[j]= NULL;
   }
