@@ -30,6 +30,7 @@
 #include "wsrep_utils.h"
 #include "wsrep_xid.h"
 #include "wsrep_thd.h"
+#include "wsrep_mysqld.h"
 
 #include <cstdio>
 #include <cstdlib>
@@ -247,7 +248,7 @@ static bool sst_auth_real_set (const char* value)
 
   if (value)
   {
-    v= my_strdup(value, MYF(0));
+    v= my_strdup(PSI_INSTRUMENT_ME, value, MYF(0));
   }
   else                                          // its NULL
   {
@@ -265,7 +266,7 @@ static bool sst_auth_real_set (const char* value)
     if (strlen(sst_auth_real))
     {
       if (wsrep_sst_auth) { my_free((void*) wsrep_sst_auth); }
-      wsrep_sst_auth= my_strdup(WSREP_SST_AUTH_MASK, MYF(0));
+      wsrep_sst_auth= my_strdup(PSI_INSTRUMENT_ME, WSREP_SST_AUTH_MASK, MYF(0));
     }
     else
     {
@@ -389,8 +390,8 @@ bool wsrep_sst_received (THD*                thd,
                                wsrep::seqno(seqno));
 
     if (!wsrep_before_SE()) {
-      wsrep_set_SE_checkpoint(wsrep::gtid::undefined());
-      wsrep_set_SE_checkpoint(sst_gtid);
+      wsrep_set_SE_checkpoint(wsrep::gtid::undefined(), wsrep_gtid_server.undefined());
+      wsrep_set_SE_checkpoint(sst_gtid, wsrep_gtid_server.gtid());
     }
     wsrep_verify_SE_checkpoint(uuid, seqno);
 
@@ -402,7 +403,7 @@ bool wsrep_sst_received (THD*                thd,
       wsrep_store_threadvars(thd);
     }
     else {
-      my_pthread_setspecific_ptr(THR_THD, NULL);
+      set_current_thd(nullptr);
     }
 
     /* During sst WSREP(thd) is not yet set for joiner. */
@@ -455,7 +456,7 @@ static char* generate_name_value(const char* name, const char* value)
   size_t name_len= strlen(name);
   size_t value_len= strlen(value);
   char* buf=
-    (char*) my_malloc((name_len + value_len + 5) * sizeof(char), MYF(0));
+    (char*) my_malloc(PSI_INSTRUMENT_ME, (name_len + value_len + 5), MYF(0));
   if (buf)
   {
     char* ref= buf;
@@ -490,11 +491,11 @@ static int generate_binlog_opt_val(char** ret)
     *ret= strcmp(opt_bin_logname, "0") ?
       generate_name_value(WSREP_SST_OPT_BINLOG,
                           opt_bin_logname) :
-      my_strdup("", MYF(0));
+      my_strdup(PSI_INSTRUMENT_ME, "", MYF(0));
   }
   else
   {
-    *ret= my_strdup("", MYF(0));
+    *ret= my_strdup(PSI_INSTRUMENT_ME, "", MYF(0));
   }
   if (!*ret) return -ENOMEM;
   return 0;
@@ -509,11 +510,11 @@ static int generate_binlog_index_opt_val(char** ret)
     *ret= strcmp(opt_binlog_index_name, "0") ?
       generate_name_value(WSREP_SST_OPT_BINLOG_INDEX,
                           opt_binlog_index_name) :
-      my_strdup("", MYF(0));
+      my_strdup(PSI_INSTRUMENT_ME, "", MYF(0));
   }
   else
   {
-    *ret= my_strdup("", MYF(0));
+    *ret= my_strdup(PSI_INSTRUMENT_ME, "", MYF(0));
   }
   if (!*ret) return -ENOMEM;
   return 0;
@@ -635,7 +636,8 @@ static void* sst_joiner_thread (void* a)
             err= EINVAL;
             goto err;
           } else {
-            wsrep_gtid_domain_id= (uint32) domain_id;
+            wsrep_gtid_server.domain_id= (uint32) domain_id;
+            wsrep_gtid_domain_id= (uint32)domain_id;
           }
         }
       }
@@ -729,7 +731,7 @@ static int sst_append_env_var(wsp::env&   env,
   return -env.error();
 }
 
-#ifdef __WIN__
+#ifdef _WIN32
 /*
   Space, single quote, ampersand, backquote, I/O redirection
   characters, caret, all brackets, plus, exclamation and comma
@@ -803,7 +805,7 @@ static size_t estimate_cmd_len (bool* extra_args)
         else if (IS_REQ_ESCAPING(c))
         {
           cmd_len++;
-#ifdef __WIN__
+#ifdef _WIN32
           quotation= true;
 #endif
         }
@@ -832,7 +834,7 @@ static size_t estimate_cmd_len (bool* extra_args)
             else if (IS_REQ_ESCAPING(c))
             {
               cmd_len++;
-#ifdef __WIN__
+#ifdef _WIN32
               quotation= true;
 #endif
             }
@@ -888,7 +890,7 @@ static void copy_orig_argv (char* cmd_str)
         else if (IS_REQ_ESCAPING(c))
         {
           plain= false;
-#ifdef __WIN__
+#ifdef _WIN32
           quotation= true;
 #endif
         }
@@ -928,7 +930,7 @@ static void copy_orig_argv (char* cmd_str)
                 c = *arg++;
                 if (IS_REQ_ESCAPING(c))
                 {
-#ifdef __WIN__
+#ifdef _WIN32
                   *cmd_str++ = c;
 #else
                   *cmd_str++ = '\\';
@@ -968,7 +970,7 @@ static void copy_orig_argv (char* cmd_str)
             else if (IS_REQ_ESCAPING(c))
             {
               plain= false;
-#ifdef __WIN__
+#ifdef _WIN32
               quotation= true;
 #endif
             }
@@ -999,7 +1001,7 @@ static void copy_orig_argv (char* cmd_str)
           {
             if (IS_REQ_ESCAPING(c))
             {
-#ifdef __WIN__
+#ifdef _WIN32
               *cmd_str++ = c;
 #else
               *cmd_str++ = '\\';
@@ -1456,13 +1458,15 @@ static int sst_donate_mysqldump (const char*         addr,
                      WSREP_SST_OPT_LPORT " '%u' "
                      WSREP_SST_OPT_SOCKET " '%s' "
                      "%s"
-                     WSREP_SST_OPT_GTID " '%s:%lld' "
+                     WSREP_SST_OPT_GTID " '%s:%lld,%d-%d-%llu' "
                      WSREP_SST_OPT_GTID_DOMAIN_ID " '%d'"
                      "%s",
                      addr, port, mysqld_port, mysqld_unix_port,
                      wsrep_defaults_file,
                      uuid_oss.str().c_str(), gtid.seqno().get(),
-                     wsrep_gtid_domain_id,
+                     wsrep_gtid_server.domain_id, wsrep_gtid_server.server_id,
+                     wsrep_gtid_server.seqno(),
+                     wsrep_gtid_server.domain_id,
                      bypass ? " " WSREP_SST_OPT_BYPASS : "");
 
   if (ret < 0 || size_t(ret) >= cmd_len)
@@ -1484,6 +1488,8 @@ static int sst_donate_mysqldump (const char*         addr,
                                         wsrep::seqno::undefined()));
   Wsrep_server_state::instance().sst_sent(sst_sent_gtid, ret);
 
+  wsrep_donor_monitor_end();
+
   return ret;
 }
 
@@ -1548,8 +1554,7 @@ static int run_sql_command(THD *thd, const char *query)
     return -1;
   }
 
-  mysql_parse(thd, thd->query(), thd->query_length(), &ps, FALSE, FALSE);
-
+  mysql_parse(thd, thd->query(), thd->query_length(), &ps);
   if (thd->is_error())
   {
     int const err= thd->get_stmt_da()->sql_errno();
@@ -1579,10 +1584,10 @@ static int sst_flush_tables(THD* thd)
   if (!is_supported_parser_charset(current_charset))
   {
       /* Do not use non-supported parser character sets */
-      WSREP_WARN("Current client character set is non-supported parser character set: %s", current_charset->csname);
+      WSREP_WARN("Current client character set is non-supported parser character set: %s", current_charset->cs_name.str);
       thd->variables.character_set_client= &my_charset_latin1;
       WSREP_WARN("For SST temporally setting character set to : %s",
-                 my_charset_latin1.csname);
+                 my_charset_latin1.cs_name.str);
   }
 
   if (run_sql_command(thd, "FLUSH TABLES WITH READ LOCK"))
@@ -1627,7 +1632,7 @@ static int sst_flush_tables(THD* thd)
     // Create a file with cluster state ID and wsrep_gtid_domain_id.
     char content[100];
     snprintf(content, sizeof(content), "%s:%lld %d\n", wsrep_cluster_state_uuid,
-             (long long)wsrep_locked_seqno, wsrep_gtid_domain_id);
+             (long long)wsrep_locked_seqno, wsrep_gtid_server.domain_id);
     err= sst_create_file(flush_success, content);
 
     if (err)
@@ -1655,7 +1660,7 @@ static int sst_flush_tables(THD* thd)
 
       fprintf(file, "%s:%lld %u\n",
               uuid_oss.str().c_str(), server_state.pause_seqno().get(),
-              wsrep_gtid_domain_id);
+              wsrep_gtid_server.domain_id);
       fsync(fileno(file));
       fclose(file);
       if (rename(tmp_name, real_name) == -1)
@@ -1861,7 +1866,7 @@ static int sst_donate_other (const char*        method,
                  method, addr, mysqld_port, mysqld_unix_port,
                  mysql_real_data_home,
                  wsrep_defaults_file,
-                 uuid_oss.str().c_str(), gtid.seqno().get(), wsrep_gtid_domain_id,
+                 uuid_oss.str().c_str(), gtid.seqno().get(), wsrep_gtid_server.domain_id,
                  binlog_opt_val, binlog_index_opt_val,
                  bypass ? " " WSREP_SST_OPT_BYPASS : "");
 
