@@ -1,4 +1,4 @@
-/* Copyright (c) 2008, 2015, Oracle and/or its affiliates. All rights reserved.
+/* Copyright (c) 2008, 2021, Oracle and/or its affiliates.
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License, version 2.0,
@@ -175,7 +175,7 @@ inline uint randomized_index(const void *ptr, uint max_size)
   seed2= seed1*seed1;
   seed1= result;
 
-  DBUG_ASSERT(result < max_size);
+  assert(result < max_size);
   return result;
 }
 
