@@ -22,8 +22,10 @@
 
 #include "strings_def.h"
 #include <m_ctype.h>
+#include "ctype-simple.h"
 
 const char charset_name_binary[]= "binary";
+#define charset_name_binary_length (sizeof(charset_name_binary)-1)
 
 static const uchar ctype_bin[]=
 {
@@ -519,7 +521,9 @@ MY_COLLATION_HANDLER my_collation_8bit_bin_handler =
   my_strcasecmp_bin,
   my_instr_bin,
   my_hash_sort_8bit_bin,
-  my_propagate_simple
+  my_propagate_simple,
+  my_min_str_8bit_simple,
+  my_max_str_8bit_simple
 };
 
 
@@ -536,7 +540,9 @@ MY_COLLATION_HANDLER my_collation_8bit_nopad_bin_handler =
   my_strcasecmp_bin,
   my_instr_bin,
   my_hash_sort_bin,
-  my_propagate_simple
+  my_propagate_simple,
+  my_min_str_8bit_simple_nopad,
+  my_max_str_8bit_simple
 };
 
 
@@ -553,7 +559,9 @@ static MY_COLLATION_HANDLER my_collation_binary_handler =
   my_strcasecmp_bin,
   my_instr_bin,
   my_hash_sort_bin,
-  my_propagate_simple
+  my_propagate_simple,
+  my_min_str_8bit_simple_nopad,
+  my_max_str_8bit_simple
 };
 
 
@@ -595,8 +603,8 @@ struct charset_info_st my_charset_bin =
 {
     63,0,0,			/* number        */
     MY_CS_COMPILED|MY_CS_BINSORT|MY_CS_PRIMARY|MY_CS_NOPAD,/* state */
-    charset_name_binary,	/* cs name    */
-    "binary",			/* name          */
+    { charset_name_binary, charset_name_binary_length},	/* cs name    */
+    { STRING_WITH_LEN("binary")},	                /* name          */
     "",				/* comment       */
     NULL,			/* tailoring     */
     ctype_bin,			/* ctype         */
