@@ -130,8 +130,9 @@ static int apply_fragment(wsrep::server_state& server_state,
         if (!apply_err)
         {
             high_priority_service.debug_crash("crash_apply_cb_before_append_frag");
+            const wsrep::xid xid(streaming_applier->transaction().xid());
             ret = high_priority_service.append_fragment_and_commit(
-                ws_handle, ws_meta, data);
+                ws_handle, ws_meta, data, xid);
             high_priority_service.debug_crash("crash_apply_cb_after_append_frag");
             ret = ret || (high_priority_service.after_apply(), 0);
         }
@@ -148,7 +149,6 @@ static int apply_fragment(wsrep::server_state& server_state,
     return ret;
 }
 
-
 static int commit_fragment(wsrep::server_state& server_state,
                            wsrep::high_priority_service& high_priority_service,
                            wsrep::high_priority_service* streaming_applier,
@@ -177,11 +177,20 @@ static int commit_fragment(wsrep::server_state& server_state,
         {
             assert(err.size() == 0);
         }
-        streaming_applier->debug_crash(
-            "crash_apply_cb_before_fragment_removal");
-        ret = ret || streaming_applier->remove_fragments(ws_meta);
-        streaming_applier->debug_crash(
-            "crash_apply_cb_after_fragment_removal");
+
+        const wsrep::transaction& trx(streaming_applier->transaction());
+        // Fragment removal for XA is going to happen in after_commit
+        if (trx.state() != wsrep::transaction::s_prepared)
+        {
+            streaming_applier->debug_crash(
+                "crash_apply_cb_before_fragment_removal");
+
+            ret = ret || streaming_applier->remove_fragments(ws_meta);
+
+            streaming_applier->debug_crash(
+                "crash_apply_cb_after_fragment_removal");
+        }
+
         streaming_applier->debug_crash(
             "crash_commit_cb_before_last_fragment_commit");
         ret = ret || streaming_applier->commit(ws_handle, ws_meta);
@@ -207,12 +216,14 @@ static int rollback_fragment(wsrep::server_state& server_state,
                              const wsrep::ws_meta& ws_meta)
 {
     int ret(0);
-    // Adopts transaction state and starts a transaction for
-    // high priority service
-    int adopt_error;
-    bool remove_fragments(false);
-    if ((adopt_error = high_priority_service.adopt_transaction(
-             streaming_applier->transaction())))
+    int adopt_error(0);
+    bool const remove_fragments(streaming_applier->transaction().
+                                streaming_context().fragments().size() > 0);
+    // If fragment removal is needed, adopt transaction state
+    // and start a transaction for it.
+    if (remove_fragments &&
+        (adopt_error = high_priority_service.adopt_transaction(
+          streaming_applier->transaction())))
     {
         log_adopt_error(streaming_applier->transaction());
     }
@@ -225,8 +236,6 @@ static int rollback_fragment(wsrep::server_state& server_state,
             high_priority_service, *streaming_applier);
         // Streaming applier rolls back out of order. Fragment
         // removal grabs commit order below.
-        remove_fragments = streaming_applier->transaction().
-                           streaming_context().fragments().size() > 0;
         ret = streaming_applier->rollback(wsrep::ws_handle(), wsrep::ws_meta());
         ret = ret || (streaming_applier->after_apply(), 0);
     }
@@ -242,7 +251,11 @@ static int rollback_fragment(wsrep::server_state& server_state,
             {
                 ret = high_priority_service.remove_fragments(ws_meta);
                 ret = ret || high_priority_service.commit(ws_handle, ws_meta);
-                ret = ret || (high_priority_service.after_apply(), 0);
+                if (ret)
+                {
+                    high_priority_service.rollback(ws_handle, ws_meta);
+                }
+                high_priority_service.after_apply();
             }
             else
             {
@@ -350,7 +363,8 @@ static int apply_write_set(wsrep::server_state& server_state,
                              ws_meta,
                              data);
     }
-    else if (ws_meta.flags() == 0)
+    else if (ws_meta.flags() == 0 || ws_meta.flags() == wsrep::provider::flag::pa_unsafe ||
+             wsrep::prepares_transaction(ws_meta.flags()))
     {
         wsrep::high_priority_service* sa(
             server_state.find_streaming_applier(
@@ -371,6 +385,7 @@ static int apply_write_set(wsrep::server_state& server_state,
         }
         else
         {
+            sa->next_fragment(ws_meta);
             ret = apply_fragment(server_state,
                                  high_priority_service,
                                  sa,
@@ -412,6 +427,7 @@ static int apply_write_set(wsrep::server_state& server_state,
             else
             {
                 // Commit fragment consumes sa
+                sa->next_fragment(ws_meta);
                 ret = commit_fragment(server_state,
                                       high_priority_service,
                                       sa,
@@ -427,7 +443,7 @@ static int apply_write_set(wsrep::server_state& server_state,
     }
     if (ret)
     {
-        wsrep::log_info() << "Failed to apply write set: " << ws_meta;
+        wsrep::log_error() << "Failed to apply write set: " << ws_meta;
     }
     return ret;
 }
@@ -452,13 +468,20 @@ static int apply_toi(wsrep::provider& provider,
     }
     else if (wsrep::starts_transaction(ws_meta.flags()))
     {
-        // NBO begin
-        throw wsrep::not_implemented_error();
+        provider.commit_order_enter(ws_handle, ws_meta);
+        wsrep::mutable_buffer err;
+        int const apply_err(high_priority_service.apply_nbo_begin(ws_meta, data, err));
+        int const vote_err(provider.commit_order_leave(ws_handle, ws_meta, err));
+        return resolve_return_error(err.size() > 0, vote_err, apply_err);
     }
     else if (wsrep::commits_transaction(ws_meta.flags()))
     {
-        // NBO end
-        throw wsrep::not_implemented_error();
+        // NBO end event is ignored here, both local and applied
+        // have NBO end handled via local TOI calls.
+        provider.commit_order_enter(ws_handle, ws_meta);
+        wsrep::mutable_buffer err;
+        provider.commit_order_leave(ws_handle, ws_meta, err);
+        return 0;
     }
     else
     {
@@ -471,15 +494,17 @@ static int apply_toi(wsrep::provider& provider,
 //                            Server State                                  //
 //////////////////////////////////////////////////////////////////////////////
 
-int wsrep::server_state::load_provider(const std::string& provider_spec,
-                                       const std::string& provider_options)
+int wsrep::server_state::load_provider(
+    const std::string& provider_spec, const std::string& provider_options,
+    const wsrep::provider::services& services)
 {
     wsrep::log_info() << "Loading provider " << provider_spec
                       << " initial position: " << initial_position_;
 
     provider_ = wsrep::provider::make_provider(*this,
                                                provider_spec,
-                                               provider_options);
+                                               provider_options,
+                                               services);
     return (provider_ ? 0 : 1);
 }
 
@@ -504,6 +529,15 @@ int wsrep::server_state::disconnect()
 {
     {
         wsrep::unique_lock<wsrep::mutex> lock(mutex_);
+        // In case of failure situations which are caused by provider
+        // being shut down some failing operation may also try to shut
+        // down the replication. Check the state here and
+        // return success if the provider disconnect is already in progress
+        // or has completed.
+        if (state(lock) == s_disconnecting || state(lock) == s_disconnected)
+        {
+            return 0;
+        }
         state(lock, s_disconnecting);
         interrupt_state_waiters(lock);
     }
@@ -571,7 +605,7 @@ wsrep::seqno wsrep::server_state::desync_and_pause()
         // Desync may give transient error if the provider cannot
         // communicate with the rest of the cluster. However, this
         // error can be tolerated because if the provider can be
-        // paused succesfully below.
+        // paused successfully below.
         WSREP_LOG_DEBUG(wsrep::log::debug_log_level(),
                         wsrep::log::debug_level_server_state,
                         "Failed to desync server before pause");
@@ -638,8 +672,10 @@ int wsrep::server_state::start_sst(const std::string& sst_request,
     if (server_service_.start_sst(sst_request, gtid, bypass))
     {
         lock.lock();
-        wsrep::log_warning() << "SST start failed";
-        state(lock, s_synced);
+        wsrep::log_warning() << "SST preparation failed";
+        // v26 API does not have JOINED event, so in anticipation of SYNCED
+        // we must do it here.
+        state(lock, s_joined);
         ret = 1;
     }
     return ret;
@@ -653,12 +689,16 @@ void wsrep::server_state::sst_sent(const wsrep::gtid& gtid, int error)
         wsrep::log_info() << "SST sending failed: " << error;
 
     wsrep::unique_lock<wsrep::mutex> lock(mutex_);
+    // v26 API does not have JOINED event, so in anticipation of SYNCED
+    // we must do it here.
     state(lock, s_joined);
     lock.unlock();
-    if (provider().sst_sent(gtid, error))
+    enum provider::status const retval(provider().sst_sent(gtid, error));
+    if (retval != provider::success)
     {
-        server_service_.log_message(wsrep::log::warning,
-                                    "Provider sst_sent() returned an error");
+        std::string msg("wsrep::sst_sent() returned an error: ");
+        msg += wsrep::provider::to_string(retval);
+        server_service_.log_message(wsrep::log::warning, msg.c_str());
     }
 }
 
@@ -687,7 +727,6 @@ void wsrep::server_state::sst_received(wsrep::client_service& cs,
                 assert(init_initialized_);
             }
         }
-        state(lock, s_joined);
         lock.unlock();
 
         if (id_.is_undefined())
@@ -699,6 +738,16 @@ void wsrep::server_state::sst_received(wsrep::client_service& cs,
 
         gtid = server_service_.get_position(cs);
         wsrep::log_info() << "Recovered position from storage: " << gtid;
+
+        lock.lock();
+        if (gtid.seqno() >= connected_gtid().seqno())
+        {
+            /* Now the node has all the data the cluster has: part in
+             * storage, part in replication event queue. */
+            state(lock, s_joined);
+        }
+        lock.unlock();
+
         wsrep::view const v(server_service_.get_view(cs, id_));
         wsrep::log_info() << "Recovered view from SST:\n" << v;
 
@@ -722,7 +771,7 @@ void wsrep::server_state::sst_received(wsrep::client_service& cs,
                  * logged view. */
                 std::ostringstream msg;
                 msg << "SST script passed bogus GTID: " << gtid
-                    << ". Preceeding view GTID: " << v.state_id();
+                    << ". Preceding view GTID: " << v.state_id();
                 throw wsrep::runtime_error(msg.str());
             }
 
@@ -747,9 +796,12 @@ void wsrep::server_state::sst_received(wsrep::client_service& cs,
         lock.unlock();
     }
 
-    if (provider().sst_received(gtid, error))
+    enum provider::status const retval(provider().sst_received(gtid, error));
+    if (retval != provider::success)
     {
-        throw wsrep::runtime_error("wsrep::sst_received() failed");
+        std::string msg("wsrep::sst_received() failed: ");
+        msg += wsrep::provider::to_string(retval);
+        throw wsrep::runtime_error(msg);
     }
 }
 
@@ -769,21 +821,6 @@ void wsrep::server_state::initialized()
     }
 }
 
-void wsrep::server_state::last_committed_gtid(const wsrep::gtid& gtid)
-{
-    wsrep::unique_lock<wsrep::mutex> lock(mutex_);
-    assert(last_committed_gtid_.is_undefined() ||
-           last_committed_gtid_.seqno() + 1 == gtid.seqno());
-    last_committed_gtid_ = gtid;
-    cond_.notify_all();
-}
-
-wsrep::gtid wsrep::server_state::last_committed_gtid() const
-{
-    wsrep::unique_lock<wsrep::mutex> lock(mutex_);
-    return last_committed_gtid_;
-}
-
 enum wsrep::provider::status
 wsrep::server_state::wait_for_gtid(const wsrep::gtid& gtid, int timeout)
     const
@@ -797,8 +834,15 @@ wsrep::server_state::set_encryption_key(std::vector<unsigned char>& key)
     encryption_key_ = key;
     if (state_ != s_disconnected)
     {
-        return provider_->enc_set_key(wsrep::const_buffer(encryption_key_.data(),
-                                                          encryption_key_.size()));
+        wsrep::const_buffer const key(encryption_key_.data(),
+                                      encryption_key_.size());
+        enum provider::status const retval(provider_->enc_set_key(key));
+        if (retval != provider::success)
+        {
+            wsrep::log_error() << "Failed to set encryption key: "
+                               << provider::to_string(retval);
+            return 1;
+        }
     }
     return 0;
 }
@@ -824,8 +868,8 @@ void wsrep::server_state::on_connect(const wsrep::view& view)
         throw wsrep::runtime_error(os.str());
     }
 
-    if (id_.is_undefined() == false &&
-        id_ != view.members()[view.own_index()].id())
+    const size_t own_index(static_cast<size_t>(view.own_index()));
+    if (id_.is_undefined() == false && id_ != view.members()[own_index].id())
     {
         std::ostringstream os;
         os << "Connection in connected state.\n"
@@ -840,7 +884,7 @@ void wsrep::server_state::on_connect(const wsrep::view& view)
     }
     else
     {
-        id_ = view.members()[view.own_index()].id();
+        id_ = view.members()[own_index].id();
     }
 
     wsrep::log_info() << "Server "
@@ -856,7 +900,7 @@ void wsrep::server_state::on_connect(const wsrep::view& view)
 }
 
 void wsrep::server_state::on_primary_view(
-    const wsrep::view& view WSREP_UNUSED,
+    const wsrep::view& view,
     wsrep::high_priority_service* high_priority_service)
 {
     wsrep::unique_lock<wsrep::mutex> lock(mutex_);
@@ -893,15 +937,8 @@ void wsrep::server_state::on_primary_view(
                 // If server side has already been initialized,
                 // skip directly to s_joined.
                 state(lock, s_initialized);
-                state(lock, s_joined);
             }
         }
-        else if (state_ == s_joiner)
-        {
-            // Got partiioned from the cluster, got IST and
-            // started applying actions.
-            state(lock, s_joined);
-        }
     }
     else
     {
@@ -909,14 +946,7 @@ void wsrep::server_state::on_primary_view(
         {
             state(lock, s_joiner);
         }
-        if (init_initialized_ && state_ != s_joined)
-        {
-            // If server side has already been initialized,
-            // skip directly to s_joined.
-            state(lock, s_joined);
-        }
     }
-
     if (init_initialized_ == false)
     {
         lock.unlock();
@@ -941,27 +971,11 @@ void wsrep::server_state::on_primary_view(
         close_orphaned_sr_transactions(lock, *high_priority_service);
     }
 
-    if (server_service_.sst_before_init())
+    if (state(lock) < s_joined &&
+        view.state_id().seqno() >= connected_gtid().seqno())
     {
-        if (state_ == s_initialized)
-        {
-            state(lock, s_joined);
-            if (init_synced_)
-            {
-                state(lock, s_synced);
-            }
-        }
-    }
-    else
-    {
-        if (state_ == s_joiner)
-        {
-            state(lock, s_joined);
-            if (init_synced_)
-            {
-                state(lock, s_synced);
-            }
-        }
+        // If we progressed beyond connected seqno, it means we have full state
+        state(lock, s_joined);
     }
 }
 
@@ -1042,19 +1056,20 @@ void wsrep::server_state::on_sync()
         {
         case s_synced:
             break;
-        case s_connected:
-            state(lock, s_joiner);
-            // fall through
-        case s_joiner:
-            state(lock, s_initializing);
+        case s_connected:                 // Seed node path: provider becomes
+            state(lock, s_joiner);        // synced with itself before anything
+            WSREP_FALLTHROUGH;            // else. Then goes DB initialization.
+        case s_joiner:                    // |
+            state(lock, s_initializing);  // V
             break;
         case s_donor:
+            assert(false); // this should never happen
             state(lock, s_joined);
             state(lock, s_synced);
             break;
         case s_initialized:
             state(lock, s_joined);
-            // fall through
+            WSREP_FALLTHROUGH;
         default:
             /* State */
             state(lock, s_synced);
@@ -1071,6 +1086,14 @@ void wsrep::server_state::on_sync()
         }
     }
     init_synced_ = true;
+
+    enum wsrep::provider::status status(send_pending_rollback_events(lock));
+    if (status)
+    {
+        // TODO should be retried?
+        wsrep::log_warning()
+            << "Failed to flush rollback event cache: " << status;
+    }
 }
 
 int wsrep::server_state::on_apply(
@@ -1244,6 +1267,23 @@ wsrep::high_priority_service* wsrep::server_state::find_streaming_applier(
     return (i == streaming_appliers_.end() ? 0 : i->second);
 }
 
+wsrep::high_priority_service* wsrep::server_state::find_streaming_applier(
+    const wsrep::xid& xid) const
+{
+    wsrep::unique_lock<wsrep::mutex> lock(mutex_);
+    streaming_appliers_map::const_iterator i(streaming_appliers_.begin());
+    while (i != streaming_appliers_.end())
+    {
+        wsrep::high_priority_service* sa(i->second);
+        if (sa->transaction().xid() == xid)
+        {
+            return sa;
+        }
+        i++;
+    }
+    return NULL;
+}
+
 //////////////////////////////////////////////////////////////////////////////
 //                              Private                                     //
 //////////////////////////////////////////////////////////////////////////////
@@ -1290,14 +1330,14 @@ void wsrep::server_state::state(
     assert(lock.owns_lock());
     static const char allowed[n_states_][n_states_] =
         {
-            /* dis, ing, ized, cted, jer, jed, dor, sed, ding */
+            /* dis, ing, ized, cted, jer, jed, dor, sed, ding to/from */
             {  0,   1,   0,    1,    0,   0,   0,   0,   0}, /* dis */
             {  1,   0,   1,    0,    0,   0,   0,   0,   1}, /* ing */
             {  1,   0,   0,    1,    0,   1,   0,   0,   1}, /* ized */
             {  1,   0,   0,    1,    1,   0,   0,   1,   1}, /* cted */
             {  1,   1,   0,    0,    0,   1,   0,   0,   1}, /* jer */
             {  1,   0,   0,    1,    0,   0,   1,   1,   1}, /* jed */
-            {  1,   0,   0,    1,    0,   1,   0,   1,   1}, /* dor */
+            {  1,   0,   0,    1,    0,   1,   0,   0,   1}, /* dor */
             {  1,   0,   0,    1,    0,   1,   1,   0,   1}, /* sed */
             {  1,   0,   0,    0,    0,   0,   0,   0,   0}  /* ding */
         };
@@ -1368,6 +1408,20 @@ void wsrep::server_state::recover_streaming_appliers_if_not_recovered(
     streaming_appliers_recovered_ = true;
 }
 
+class transaction_state_cmp
+{
+public:
+    transaction_state_cmp(const enum wsrep::transaction::state s)
+        : state_(s) { }
+    bool operator()(const std::pair<wsrep::client_id,
+                    wsrep::client_state*>& vt) const
+    {
+        return vt.second->transaction().state() == state_;
+    }
+private:
+    enum wsrep::transaction::state state_;
+};
+
 void wsrep::server_state::close_orphaned_sr_transactions(
     wsrep::unique_lock<wsrep::mutex>& lock,
     wsrep::high_priority_service& high_priority_service)
@@ -1383,14 +1437,22 @@ void wsrep::server_state::close_orphaned_sr_transactions(
     // - (1 non-primary) and (2 non-primary)
     // - (1,2 primary)
     // We need to rollback SRs owned by both 1 and 2.
+    // Notice that since the introduction of rollback_event_queue_,
+    // checking for equal consecutive views is no longer needed.
+    // However, we must keep it here for the time being, for backwards
+    // compatibility.
     const bool equal_consecutive_views =
         current_view_.equal_membership(previous_primary_view_);
 
     if (current_view_.own_index() == -1 || equal_consecutive_views)
     {
-        while (streaming_clients_.empty() == false)
+        streaming_clients_map::iterator i;
+        transaction_state_cmp prepared_state_cmp(wsrep::transaction::s_prepared);
+        while ((i = std::find_if_not(streaming_clients_.begin(),
+                                     streaming_clients_.end(),
+                                     prepared_state_cmp))
+               != streaming_clients_.end())
         {
-            streaming_clients_map::iterator i(streaming_clients_.begin());
             wsrep::client_id client_id(i->first);
             wsrep::transaction_id transaction_id(i->second->transaction().id());
             // It is safe to unlock the server state temporarily here.
@@ -1418,13 +1480,17 @@ void wsrep::server_state::close_orphaned_sr_transactions(
     streaming_appliers_map::iterator i(streaming_appliers_.begin());
     while (i != streaming_appliers_.end())
     {
-        // rollback SR on equal consecutive primary views or if its
-        // originator is not in the current view
-        if (equal_consecutive_views ||
-            (std::find_if(current_view_.members().begin(),
-                          current_view_.members().end(),
-                          server_id_cmp(i->first.first)) ==
-             current_view_.members().end()))
+        wsrep::high_priority_service* streaming_applier(i->second);
+
+        // Rollback SR on equal consecutive primary views or if its
+        // originator is not in the current view.
+        // Transactions in prepared state must be committed or
+        // rolled back explicitly, those are never rolled back here.
+        if ((streaming_applier->transaction().state() !=
+             wsrep::transaction::s_prepared) &&
+            (equal_consecutive_views ||
+             not current_view_.is_member(
+                 streaming_applier->transaction().server_id())))
         {
             WSREP_LOG_DEBUG(wsrep::log::debug_log_level(),
                             wsrep::log::debug_level_server_state,
@@ -1433,7 +1499,6 @@ void wsrep::server_state::close_orphaned_sr_transactions(
                             << ", " << i->first.second);
             wsrep::id server_id(i->first.first);
             wsrep::transaction_id transaction_id(i->first.second);
-            wsrep::high_priority_service* streaming_applier(i->second);
             int adopt_error;
             if ((adopt_error = high_priority_service.adopt_transaction(
                      streaming_applier->transaction())))
@@ -1498,3 +1563,50 @@ void wsrep::server_state::close_transactions_at_disconnect(
     }
     streaming_appliers_recovered_ = false;
 }
+
+//
+// Rollback event queue
+//
+
+void wsrep::server_state::queue_rollback_event(
+    const wsrep::transaction_id& id)
+{
+    wsrep::unique_lock<wsrep::mutex> lock(mutex_);
+#ifndef NDEBUG
+    // Make sure we don't have duplicate
+    // transaction ids in rollback event queue.
+    // There is no need to do this in release
+    // build given that caller (streaming_rollback())
+    // should avoid duplicates.
+    for (auto i : rollback_event_queue_)
+    {
+        assert(id != i);
+    }
+#endif
+    rollback_event_queue_.push_back(id);
+}
+
+enum wsrep::provider::status
+wsrep::server_state::send_pending_rollback_events(
+    wsrep::unique_lock<wsrep::mutex>& lock WSREP_UNUSED)
+{
+    assert(lock.owns_lock());
+    while (not rollback_event_queue_.empty())
+    {
+        const wsrep::transaction_id& id(rollback_event_queue_.front());
+        const enum wsrep::provider::status status(provider().rollback(id));
+        if (status)
+        {
+            return status;
+        }
+        rollback_event_queue_.pop_front();
+    }
+    return wsrep::provider::success;
+}
+
+enum wsrep::provider::status
+wsrep::server_state::send_pending_rollback_events()
+{
+    wsrep::unique_lock<wsrep::mutex> lock(mutex_);
+    return send_pending_rollback_events(lock);
+}
